@@ -25,13 +25,15 @@ import os
 import socket
 import time
 import threading
+# TODO: switch as much of py3compat.py to 'six' as possible, then use six.wraps
 from functools import wraps
 
 from paramiko import util
-from paramiko.common import cMSG_CHANNEL_REQUEST, cMSG_CHANNEL_WINDOW_ADJUST, \
-    cMSG_CHANNEL_DATA, cMSG_CHANNEL_EXTENDED_DATA, DEBUG, ERROR, \
-    cMSG_CHANNEL_SUCCESS, cMSG_CHANNEL_FAILURE, cMSG_CHANNEL_EOF, \
-    cMSG_CHANNEL_CLOSE
+from paramiko.common import (
+    cMSG_CHANNEL_REQUEST, cMSG_CHANNEL_WINDOW_ADJUST, cMSG_CHANNEL_DATA,
+    cMSG_CHANNEL_EXTENDED_DATA, DEBUG, ERROR, cMSG_CHANNEL_SUCCESS,
+    cMSG_CHANNEL_FAILURE, cMSG_CHANNEL_EOF, cMSG_CHANNEL_CLOSE,
+)
 from paramiko.message import Message
 from paramiko.py3compat import bytes_types
 from paramiko.ssh_exception import SSHException
@@ -45,16 +47,17 @@ def open_only(func):
     """
     Decorator for `.Channel` methods which performs an openness check.
 
-    :raises SSHException:
-        If the wrapped method is called on an unopened `.Channel`.
+    :raises:
+        `.SSHException` -- If the wrapped method is called on an unopened
+        `.Channel`.
     """
     @wraps(func)
     def _check(self, *args, **kwds):
         if (
-            self.closed
-            or self.eof_received
-            or self.eof_sent
-            or not self.active
+            self.closed or
+            self.eof_received or
+            self.eof_sent or
+            not self.active
         ):
             raise SSHException('Channel is not open')
         return func(self, *args, **kwds)
@@ -74,7 +77,7 @@ class Channel (ClosingContextManager):
     flow-controlled independently.)  Similarly, if the server isn't reading
     data you send, calls to `send` may block, unless you set a timeout.  This
     is exactly like a normal network socket, so it shouldn't be too surprising.
-    
+
     Instances of this class may be used as context managers.
     """
 
@@ -155,16 +158,18 @@ class Channel (ClosingContextManager):
         after creating a client channel, to ask the server to provide some
         basic terminal semantics for a shell invoked with `invoke_shell`.
         It isn't necessary (or desirable) to call this method if you're going
-        to exectue a single command with `exec_command`.
+        to execute a single command with `exec_command`.
 
-        :param str term: the terminal type to emulate (for example, ``'vt100'``)
+        :param str term: the terminal type to emulate
+            (for example, ``'vt100'``)
         :param int width: width (in characters) of the terminal screen
         :param int height: height (in characters) of the terminal screen
         :param int width_pixels: width (in pixels) of the terminal screen
         :param int height_pixels: height (in pixels) of the terminal screen
 
-        :raises SSHException:
-            if the request was rejected or the channel was closed
+        :raises:
+            `.SSHException` -- if the request was rejected or the channel was
+            closed
         """
         m = Message()
         m.add_byte(cMSG_CHANNEL_REQUEST)
@@ -195,7 +200,8 @@ class Channel (ClosingContextManager):
         When the shell exits, the channel will be closed and can't be reused.
         You must open a new channel if you wish to open another shell.
 
-        :raises SSHException: if the request was rejected or the channel was
+        :raises:
+            `.SSHException` -- if the request was rejected or the channel was
             closed
         """
         m = Message()
@@ -220,7 +226,8 @@ class Channel (ClosingContextManager):
 
         :param str command: a shell command to execute.
 
-        :raises SSHException: if the request was rejected or the channel was
+        :raises:
+            `.SSHException` -- if the request was rejected or the channel was
             closed
         """
         m = Message()
@@ -245,8 +252,9 @@ class Channel (ClosingContextManager):
 
         :param str subsystem: name of the subsystem being requested.
 
-        :raises SSHException:
-            if the request was rejected or the channel was closed
+        :raises:
+            `.SSHException` -- if the request was rejected or the channel was
+            closed
         """
         m = Message()
         m.add_byte(cMSG_CHANNEL_REQUEST)
@@ -269,8 +277,9 @@ class Channel (ClosingContextManager):
         :param int width_pixels: new width (in pixels) of the terminal screen
         :param int height_pixels: new height (in pixels) of the terminal screen
 
-        :raises SSHException:
-            if the request was rejected or the channel was closed
+        :raises:
+            `.SSHException` -- if the request was rejected or the channel was
+            closed
         """
         m = Message()
         m.add_byte(cMSG_CHANNEL_REQUEST)
@@ -283,6 +292,59 @@ class Channel (ClosingContextManager):
         m.add_int(height_pixels)
         self.transport._send_user_message(m)
 
+    @open_only
+    def update_environment(self, environment):
+        """
+        Updates this channel's remote shell environment.
+
+        .. note::
+            This operation is additive - i.e. the current environment is not
+            reset before the given environment variables are set.
+
+        .. warning::
+            Servers may silently reject some environment variables; see the
+            warning in `set_environment_variable` for details.
+
+        :param dict environment:
+            a dictionary containing the name and respective values to set
+        :raises:
+            `.SSHException` -- if any of the environment variables was rejected
+            by the server or the channel was closed
+        """
+        for name, value in environment.items():
+            try:
+                self.set_environment_variable(name, value)
+            except SSHException as e:
+                err = "Failed to set environment variable \"{0}\"."
+                raise SSHException(err.format(name), e)
+
+    @open_only
+    def set_environment_variable(self, name, value):
+        """
+        Set the value of an environment variable.
+
+        .. warning::
+            The server may reject this request depending on its ``AcceptEnv``
+            setting; such rejections will fail silently (which is common client
+            practice for this particular request type). Make sure you
+            understand your server's configuration before using!
+
+        :param str name: name of the environment variable
+        :param str value: value of the environment variable
+
+        :raises:
+            `.SSHException` -- if the request was rejected or the channel was
+            closed
+        """
+        m = Message()
+        m.add_byte(cMSG_CHANNEL_REQUEST)
+        m.add_int(self.remote_chanid)
+        m.add_string('env')
+        m.add_boolean(False)
+        m.add_string(name)
+        m.add_string(value)
+        self.transport._send_user_message(m)
+
     def exit_status_ready(self):
         """
         Return true if the remote process has exited and returned an exit
@@ -311,11 +373,11 @@ class Channel (ClosingContextManager):
             `.Transport` or session's ``window_size`` (e.g. that set by the
             ``default_window_size`` kwarg for `.Transport.__init__`) will cause
             `.recv_exit_status` to hang indefinitely if it is called prior to a
-            sufficiently large `~Channel..read` (or if there are no threads
-            calling `~Channel.read` in the background).
+            sufficiently large `.Channel.recv` (or if there are no threads
+            calling `.Channel.recv` in the background).
 
             In these cases, ensuring that `.recv_exit_status` is called *after*
-            `~Channel.read` (or, again, using threads) can avoid the hang.
+            `.Channel.recv` (or, again, using threads) can avoid the hang.
 
         :return: the exit code (as an `int`) of the process on the server.
 
@@ -347,8 +409,14 @@ class Channel (ClosingContextManager):
         self.transport._send_user_message(m)
 
     @open_only
-    def request_x11(self, screen_number=0, auth_protocol=None, auth_cookie=None,
-                    single_connection=False, handler=None):
+    def request_x11(
+            self,
+            screen_number=0,
+            auth_protocol=None,
+            auth_cookie=None,
+            single_connection=False,
+            handler=None
+    ):
         """
         Request an x11 session on this channel.  If the server allows it,
         further x11 requests can be made from the server to the client,
@@ -364,7 +432,7 @@ class Channel (ClosingContextManager):
         generated, used, and returned.  You will need to use this value to
         verify incoming x11 requests and replace them with the actual local
         x11 cookie (which requires some knowledge of the x11 protocol).
-        
+
         If a handler is passed in, the handler is called from another thread
         whenever a new x11 connection arrives.  The default handler queues up
         incoming x11 connections, which may be retrieved using
@@ -383,8 +451,8 @@ class Channel (ClosingContextManager):
             if True, only a single x11 connection will be forwarded (by
             default, any number of x11 connections can arrive over this
             session)
-        :param function handler:
-            an optional handler to use for incoming X11 connections
+        :param handler:
+            an optional callable handler to use for incoming X11 connections
         :return: the auth_cookie used
         """
         if auth_protocol is None:
@@ -413,10 +481,12 @@ class Channel (ClosingContextManager):
         Request for a forward SSH Agent on this channel.
         This is only valid for an ssh-agent from OpenSSH !!!
 
-        :param function handler:
-            a required handler to use for incoming SSH Agent connections
+        :param handler:
+            a required callable handler to use for incoming SSH Agent
+            connections
 
-        :return: True if we are ok, else False (at that time we always return ok)
+        :return: True if we are ok, else False
+            (at that time we always return ok)
 
         :raises: SSHException in case of channel problem.
         """
@@ -497,16 +567,16 @@ class Channel (ClosingContextManager):
             self._feed(data)
         return old
 
-    ###  socket API
+    # ...socket API...
 
     def settimeout(self, timeout):
         """
         Set a timeout on blocking read/write operations.  The ``timeout``
-        argument can be a nonnegative float expressing seconds, or ``None``.  If
-        a float is given, subsequent channel read/write operations will raise
-        a timeout exception if the timeout period value has elapsed before the
-        operation has completed.  Setting a timeout of ``None`` disables
-        timeouts on socket operations.
+        argument can be a nonnegative float expressing seconds, or ``None``.
+        If a float is given, subsequent channel read/write operations will
+        raise a timeout exception if the timeout period value has elapsed
+        before the operation has completed.  Setting a timeout of ``None``
+        disables timeouts on socket operations.
 
         ``chan.settimeout(0.0)`` is equivalent to ``chan.setblocking(0)``;
         ``chan.settimeout(None)`` is equivalent to ``chan.setblocking(1)``.
@@ -600,11 +670,11 @@ class Channel (ClosingContextManager):
         """
         Receive data from the channel.  The return value is a string
         representing the data received.  The maximum amount of data to be
-        received at once is specified by ``nbytes``.  If a string of length zero
-        is returned, the channel stream has closed.
+        received at once is specified by ``nbytes``.  If a string of
+        length zero is returned, the channel stream has closed.
 
         :param int nbytes: maximum number of bytes to read.
-        :return: received data, as a `bytes`
+        :return: received data, as a ``str``/``bytes``.
 
         :raises socket.timeout:
             if no data is ready before the timeout set by `settimeout`.
@@ -751,7 +821,7 @@ class Channel (ClosingContextManager):
             if sending stalled for longer than the timeout set by `settimeout`.
         :raises socket.error:
             if an error occurred before the entire string was sent.
-        
+
         .. note::
             If the channel is closed while only part of the data has been
             sent, there is no way to determine how much data (if any) was sent.
@@ -775,7 +845,7 @@ class Channel (ClosingContextManager):
             if sending stalled for longer than the timeout set by `settimeout`.
         :raises socket.error:
             if an error occurred before the entire string was sent.
-            
+
         .. versionadded:: 1.1
         """
         while s:
@@ -893,7 +963,7 @@ class Channel (ClosingContextManager):
         # attribute instead of a semipublic .closed attribute.
         return self.closed
 
-    ###  calls from Transport
+    # ...calls from Transport
 
     def _set_transport(self, transport):
         self.transport = transport
@@ -902,7 +972,8 @@ class Channel (ClosingContextManager):
     def _set_window(self, window_size, max_packet_size):
         self.in_window_size = window_size
         self.in_max_packet_size = max_packet_size
-        # threshold of bytes we receive before we bother to send a window update
+        # threshold of bytes we receive before we bother to send
+        # a window update
         self.in_window_threshold = window_size // 10
         self.in_window_sofar = 0
         self._log(DEBUG, 'Max packet in: %d bytes' % max_packet_size)
@@ -910,8 +981,9 @@ class Channel (ClosingContextManager):
     def _set_remote_channel(self, chanid, window_size, max_packet_size):
         self.remote_chanid = chanid
         self.out_window_size = window_size
-        self.out_max_packet_size = self.transport. \
-            _sanitize_packet_size(max_packet_size)
+        self.out_max_packet_size = self.transport._sanitize_packet_size(
+            max_packet_size
+        )
         self.active = 1
         self._log(DEBUG, 'Max packet out: %d bytes' % self.out_max_packet_size)
 
@@ -943,7 +1015,10 @@ class Channel (ClosingContextManager):
         code = m.get_int()
         s = m.get_binary()
         if code != 1:
-            self._log(ERROR, 'unknown extended_data type %d; discarding' % code)
+            self._log(
+                ERROR,
+                'unknown extended_data type %d; discarding' % code
+            )
             return
         if self.combine_stderr:
             self._feed(s)
@@ -983,8 +1058,15 @@ class Channel (ClosingContextManager):
             if server is None:
                 ok = False
             else:
-                ok = server.check_channel_pty_request(self, term, width, height, pixelwidth,
-                                                      pixelheight, modes)
+                ok = server.check_channel_pty_request(
+                    self,
+                    term,
+                    width,
+                    height,
+                    pixelwidth,
+                    pixelheight,
+                    modes
+                )
         elif key == 'shell':
             if server is None:
                 ok = False
@@ -1017,8 +1099,8 @@ class Channel (ClosingContextManager):
             if server is None:
                 ok = False
             else:
-                ok = server.check_channel_window_change_request(self, width, height, pixelwidth,
-                                                                pixelheight)
+                ok = server.check_channel_window_change_request(
+                    self, width, height, pixelwidth, pixelheight)
         elif key == 'x11-req':
             single_connection = m.get_boolean()
             auth_proto = m.get_text()
@@ -1027,8 +1109,13 @@ class Channel (ClosingContextManager):
             if server is None:
                 ok = False
             else:
-                ok = server.check_channel_x11_request(self, single_connection,
-                                                      auth_proto, auth_cookie, screen_number)
+                ok = server.check_channel_x11_request(
+                    self,
+                    single_connection,
+                    auth_proto,
+                    auth_cookie,
+                    screen_number
+                )
         elif key == 'auth-agent-req@openssh.com':
             if server is None:
                 ok = False
@@ -1070,14 +1157,15 @@ class Channel (ClosingContextManager):
             if m is not None:
                 self.transport._send_user_message(m)
 
-    ###  internals...
+    # ...internals...
 
     def _send(self, s, m):
         size = len(s)
         self.lock.acquire()
         try:
             if self.closed:
-                # this doesn't seem useful, but it is the documented behavior of Socket
+                # this doesn't seem useful, but it is the documented behavior
+                # of Socket
                 raise socket.error('Socket is closed')
             size = self._wait_for_send_window(size)
             if size == 0:
@@ -1145,7 +1233,8 @@ class Channel (ClosingContextManager):
         return m1, m2
 
     def _unlink(self):
-        # server connection could die before we become active: still signal the close!
+        # server connection could die before we become active:
+        # still signal the close!
         if self.closed:
             return
         self.lock.acquire()
@@ -1188,7 +1277,8 @@ class Channel (ClosingContextManager):
             # should we block?
             if self.timeout == 0.0:
                 raise socket.timeout()
-            # loop here in case we get woken up but a different thread has filled the buffer
+            # loop here in case we get woken up but a different thread has
+            # filled the buffer
             timeout = self.timeout
             while self.out_window_size == 0:
                 if self.closed or self.eof_sent:
