@@ -15,9 +15,10 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with Paramiko; if not, write to the Free Software Foundation, Inc.,
 # 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
-from paramiko.common import linefeed_byte_value, crlf, cr_byte, linefeed_byte, \
-    cr_byte_value
-from paramiko.py3compat import BytesIO, PY2, u, b, bytes_types
+from paramiko.common import (
+    linefeed_byte_value, crlf, cr_byte, linefeed_byte, cr_byte_value,
+)
+from paramiko.py3compat import BytesIO, PY2, u, bytes_types, text_type
 
 from paramiko.util import ClosingContextManager
 
@@ -66,7 +67,7 @@ class BufferedFile (ClosingContextManager):
         file.  This iterator happens to return the file itself, since a file is
         its own iterator.
 
-        :raises ValueError: if the file is closed.
+        :raises: ``ValueError`` -- if the file is closed.
         """
         if self._closed:
             raise ValueError('I/O operation on closed file')
@@ -92,10 +93,10 @@ class BufferedFile (ClosingContextManager):
         def next(self):
             """
             Returns the next line from the input, or raises
-            `~exceptions.StopIteration` when EOF is hit.  Unlike Python file
+            ``StopIteration`` when EOF is hit.  Unlike Python file
             objects, it's okay to mix calls to `next` and `readline`.
 
-            :raises StopIteration: when the end of the file is reached.
+            :raises: ``StopIteration`` -- when the end of the file is reached.
 
             :returns: a line (`str`) read from the file.
             """
@@ -106,11 +107,11 @@ class BufferedFile (ClosingContextManager):
     else:
         def __next__(self):
             """
-            Returns the next line from the input, or raises `.StopIteration` when
-            EOF is hit.  Unlike python file objects, it's okay to mix calls to
-            `.next` and `.readline`.
+            Returns the next line from the input, or raises ``StopIteration``
+            when EOF is hit.  Unlike python file objects, it's okay to mix
+            calls to `.next` and `.readline`.
 
-            :raises StopIteration: when the end of the file is reached.
+            :raises: ``StopIteration`` -- when the end of the file is reached.
 
             :returns: a line (`str`) read from the file.
             """
@@ -151,8 +152,8 @@ class BufferedFile (ClosingContextManager):
 
     def readinto(self, buff):
         """
-        Read up to ``len(buff)`` bytes into :class:`bytearray` *buff* and
-        return the number of bytes read.
+        Read up to ``len(buff)`` bytes into ``bytearray`` *buff* and return the
+        number of bytes read.
 
         :returns:
             The number of bytes read.
@@ -163,9 +164,9 @@ class BufferedFile (ClosingContextManager):
 
     def read(self, size=None):
         """
-        Read at most ``size`` bytes from the file (less if we hit the end of the
-        file first).  If the ``size`` argument is negative or omitted, read all
-        the remaining data in the file.
+        Read at most ``size`` bytes from the file (less if we hit the end of
+        the file first).  If the ``size`` argument is negative or omitted,
+        read all the remaining data in the file.
 
         .. note::
             ``'b'`` mode flag is ignored (``self.FLAG_BINARY`` in
@@ -250,7 +251,11 @@ class BufferedFile (ClosingContextManager):
         line = self._rbuffer
         truncated = False
         while True:
-            if self._at_trailing_cr and (self._flags & self.FLAG_UNIVERSAL_NEWLINE) and (len(line) > 0):
+            if (
+                self._at_trailing_cr and
+                self._flags & self.FLAG_UNIVERSAL_NEWLINE and
+                len(line) > 0
+            ):
                 # edge case: the newline may be '\r\n' and we may have read
                 # only the first '\r' last time.
                 if line[0] == linefeed_byte_value:
@@ -271,7 +276,13 @@ class BufferedFile (ClosingContextManager):
                 n = size - len(line)
             else:
                 n = self._bufsize
-            if (linefeed_byte in line) or ((self._flags & self.FLAG_UNIVERSAL_NEWLINE) and (cr_byte in line)):
+            if (
+                linefeed_byte in line or
+                (
+                    self._flags & self.FLAG_UNIVERSAL_NEWLINE and
+                    cr_byte in line
+                )
+            ):
                 break
             try:
                 new_data = self._read(n)
@@ -294,12 +305,20 @@ class BufferedFile (ClosingContextManager):
             self._pos += len(line)
             return line if self._flags & self.FLAG_BINARY else u(line)
         xpos = pos + 1
-        if (line[pos] == cr_byte_value) and (xpos < len(line)) and (line[xpos] == linefeed_byte_value):
+        if (
+            line[pos] == cr_byte_value and
+            xpos < len(line) and
+            line[xpos] == linefeed_byte_value
+        ):
             xpos += 1
         # if the string was truncated, _rbuffer needs to have the string after
         # the newline character plus the truncated part of the line we stored
         # earlier in _rbuffer
-        self._rbuffer = line[xpos:] + self._rbuffer if truncated else line[xpos:]
+        if truncated:
+            self._rbuffer = line[xpos:] + self._rbuffer
+        else:
+            self._rbuffer = line[xpos:]
+
         lf = line[pos:xpos]
         line = line[:pos] + linefeed_byte
         if (len(self._rbuffer) == 0) and (lf == cr_byte):
@@ -349,7 +368,7 @@ class BufferedFile (ClosingContextManager):
             type of movement: 0 = absolute; 1 = relative to the current
             position; 2 = relative to the end of the file.
 
-        :raises IOError: if the file doesn't support random access.
+        :raises: ``IOError`` -- if the file doesn't support random access.
         """
         raise IOError('File does not support seeking.')
 
@@ -370,9 +389,11 @@ class BufferedFile (ClosingContextManager):
         written yet.  (Use `flush` or `close` to force buffered data to be
         written out.)
 
-        :param str/bytes data: data to write
+        :param data: ``str``/``bytes`` data to write
         """
-        data = b(data)
+        if isinstance(data, text_type):
+            # Accept text and encode as utf-8 for compatibility only.
+            data = data.encode('utf-8')
         if self._closed:
             raise IOError('File is closed')
         if not (self._flags & self.FLAG_WRITE):
@@ -404,7 +425,7 @@ class BufferedFile (ClosingContextManager):
         name is intended to match `readlines`; `writelines` does not add line
         separators.)
 
-        :param iterable sequence: an iterable sequence of strings.
+        :param sequence: an iterable sequence of strings.
         """
         for line in sequence:
             self.write(line)
@@ -421,7 +442,7 @@ class BufferedFile (ClosingContextManager):
     def closed(self):
         return self._closed
 
-    ###  overrides...
+    # ...overrides...
 
     def _read(self, size):
         """
@@ -449,7 +470,7 @@ class BufferedFile (ClosingContextManager):
         """
         return 0
 
-    ###  internals...
+    # ...internals...
 
     def _set_mode(self, mode='r', bufsize=-1):
         """
@@ -513,7 +534,10 @@ class BufferedFile (ClosingContextManager):
             return
         if self.newlines is None:
             self.newlines = newline
-        elif self.newlines != newline and isinstance(self.newlines, bytes_types):
+        elif (
+            self.newlines != newline and
+            isinstance(self.newlines, bytes_types)
+        ):
             self.newlines = (self.newlines, newline)
         elif newline not in self.newlines:
             self.newlines += (newline,)
