@@ -22,7 +22,6 @@ Useful functions used by the rest of paramiko.
 
 from __future__ import generators
 
-import array
 import errno
 import sys
 import struct
@@ -31,12 +30,13 @@ import threading
 import logging
 
 from paramiko.common import DEBUG, zero_byte, xffffffff, max_byte
-from paramiko.py3compat import PY2, long, byte_ord, b, byte_chr
+from paramiko.py3compat import PY2, long, byte_chr, byte_ord, b
 from paramiko.config import SSHConfig
 
 
 def inflate_long(s, always_positive=False):
-    """turns a normalized byte string into a long-int (adapted from Crypto.Util.number)"""
+    """turns a normalized byte string into a long-int
+    (adapted from Crypto.Util.number)"""
     out = long(0)
     negative = 0
     if not always_positive and (len(s) > 0) and (byte_ord(s[0]) >= 0x80):
@@ -49,17 +49,19 @@ def inflate_long(s, always_positive=False):
         # noinspection PyAugmentAssignment
         s = filler * (4 - len(s) % 4) + s
     for i in range(0, len(s), 4):
-        out = (out << 32) + struct.unpack('>I', s[i:i+4])[0]
+        out = (out << 32) + struct.unpack('>I', s[i:i + 4])[0]
     if negative:
         out -= (long(1) << (8 * len(s)))
     return out
 
+
 deflate_zero = zero_byte if PY2 else 0
 deflate_ff = max_byte if PY2 else 0xff
 
 
 def deflate_long(n, add_sign_padding=True):
-    """turns a long-int into a normalized byte string (adapted from Crypto.Util.number)"""
+    """turns a long-int into a normalized byte string
+    (adapted from Crypto.Util.number)"""
     # after much testing, this algorithm was deemed to be the fastest
     s = bytes()
     n = long(n)
@@ -92,16 +94,16 @@ def format_binary(data, prefix=''):
     x = 0
     out = []
     while len(data) > x + 16:
-        out.append(format_binary_line(data[x:x+16]))
+        out.append(format_binary_line(data[x:x + 16]))
         x += 16
     if x < len(data):
         out.append(format_binary_line(data[x:]))
-    return [prefix + x for x in out]
+    return [prefix + line for line in out]
 
 
 def format_binary_line(data):
     left = ' '.join(['%02X' % byte_ord(c) for c in data])
-    right = ''.join([('.%c..' % c)[(byte_ord(c)+63)//95] for c in data])
+    right = ''.join([('.%c..' % c)[(byte_ord(c) + 63) // 95] for c in data])
     return '%-50s %s' % (left, right)
 
 
@@ -216,6 +218,7 @@ def mod_inverse(x, m):
         u2 += m
     return u2
 
+
 _g_thread_ids = {}
 _g_thread_counter = 0
 _g_thread_lock = threading.Lock()
@@ -237,15 +240,16 @@ def get_thread_id():
 
 
 def log_to_file(filename, level=DEBUG):
-    """send paramiko logs to a logfile, if they're not already going somewhere"""
+    """send paramiko logs to a logfile,
+    if they're not already going somewhere"""
     l = logging.getLogger("paramiko")
     if len(l.handlers) > 0:
         return
     l.setLevel(level)
     f = open(filename, 'a')
     lh = logging.StreamHandler(f)
-    lh.setFormatter(logging.Formatter('%(levelname)-.3s [%(asctime)s.%(msecs)03d] thr=%(_threadid)-3d %(name)s: %(message)s',
-                                      '%Y%m%d-%H:%M:%S'))
+    frm = '%(levelname)-.3s [%(asctime)s.%(msecs)03d] thr=%(_threadid)-3d %(name)s: %(message)s' # noqa
+    lh.setFormatter(logging.Formatter(frm, '%Y%m%d-%H:%M:%S'))
     l.addHandler(lh)
 
 
@@ -254,6 +258,8 @@ class PFilter (object):
     def filter(self, record):
         record._threadid = get_thread_id()
         return True
+
+
 _pfilter = PFilter()
 
 
@@ -273,43 +279,12 @@ def retry_on_signal(function):
                 raise
 
 
-class Counter (object):
-    """Stateful counter for CTR mode crypto"""
-    def __init__(self, nbits, initial_value=long(1), overflow=long(0)):
-        self.blocksize = nbits / 8
-        self.overflow = overflow
-        # start with value - 1 so we don't have to store intermediate values when counting
-        # could the iv be 0?
-        if initial_value == 0:
-            self.value = array.array('c', max_byte * self.blocksize)
-        else:
-            x = deflate_long(initial_value - 1, add_sign_padding=False)
-            self.value = array.array('c', zero_byte * (self.blocksize - len(x)) + x)
-
-    def __call__(self):
-        """Increament the counter and return the new value"""
-        i = self.blocksize - 1
-        while i > -1:
-            c = self.value[i] = byte_chr((byte_ord(self.value[i]) + 1) % 256)
-            if c != zero_byte:
-                return self.value.tostring()
-            i -= 1
-        # counter reset
-        x = deflate_long(self.overflow, add_sign_padding=False)
-        self.value = array.array('c', zero_byte * (self.blocksize - len(x)) + x)
-        return self.value.tostring()
-
-    @classmethod
-    def new(cls, nbits, initial_value=long(1), overflow=long(0)):
-        return cls(nbits, initial_value=initial_value, overflow=overflow)
-
-
 def constant_time_bytes_eq(a, b):
     if len(a) != len(b):
         return False
     res = 0
     # noinspection PyUnresolvedReferences
-    for i in (xrange if PY2 else range)(len(a)):
+    for i in (xrange if PY2 else range)(len(a)):  # noqa: F821
         res |= byte_ord(a[i]) ^ byte_ord(b[i])
     return res == 0
 
